<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2011 Kerstin Gerull <support@form4.de>, form4 GmbH & Co. KG
 *  (c) 2011 Marko Roeper <marko.roeper@googlemail.com>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/


/**
 *
 *
 * @package form4_tags
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License, version 3 or later
 *
 */
class Tx_Form4Tags_Controller_TagController extends Tx_Extbase_MVC_Controller_ActionController {

	/**
	 * tagRepository
	 *
	 * @var Tx_Form4Tags_Domain_Repository_TagRepository
	 */
	protected $tagRepository;

	/**
	 * @var tslib_fe a reference to $GLOBALS['TSFE']
	 */
	protected $_tsfe;

	public function initializeAction() {
		parent::initializeAction();

		$this->_tsfe = & $GLOBALS['TSFE'];

		$this->tagRepository->setController($this);

		$contentObject = $this->configurationManager->getContentObject();
		$this->tagRepository->setContentObject($contentObject);

		$querySettings = $this->objectManager->create('Tx_Extbase_Persistence_Typo3QuerySettings');
		$querySettings->setRespectStoragePage(false);
		$querySettings->setRespectSysLanguage(true);
		$querySettings->setRespectEnableFields(true);
		$this->tagRepository->setDefaultQuerySettings($querySettings);
	}

	/**
	 * injectTagRepository
	 *
	 * @param Tx_Form4Tags_Domain_Repository_TagRepository $tagRepository
	 * @return void
	 */
	public function injectTagRepository(Tx_Form4Tags_Domain_Repository_TagRepository $tagRepository) {
		$this->tagRepository = $tagRepository;
	}

	/**
	 * action showTeaser
	 *
	 * @return void
	 */
	public function showTeaserAction() {
		$teasers = array();
		$teaserCount = intval($this->settings['teaserCount']);
		$args = $this->request->getArguments();
// 		t3lib_div::debug($args);
		$tag = 0;
		if(!empty($args)){
			$tag = max(intval($args['tag']),0);
		}

		if($tag==0) {
			$this->view->assign('hasTeasers', false);
			return;
		}

		$contentObject = $this->configurationManager->getContentObject();
		// Tagdaten laden
		$tagData = $this->tagRepository->getTagByUid($tag);
		if (empty($tagData)) {
			$this->view->assign('hasTeasers', false);
			return;
		}

		// weitere Seiten mit diesem Tag laden
		$pages = $this->tagRepository->getPagesByTagUid(
			$tag, ($teaserCount > 0 ? $teaserCount : '')
		);

		if (empty($pages)) {
			$this->view->assign('hasTeasers', false);
			return;
		}

		$pagesUIDs = array();
		foreach($pages AS $key => $value){
			if (
				$this->_tsfe->sys_language_contentOL &&
				$this->_tsfe->sys_language_uid > 0
			) {
				if ($value['_PAGES_OVERLAY']) {
					$pagesUIDs[] = $value['uid'];
				} else {
					unset($pages[$key]);
				}
			} else {
				$pagesUIDs[] = $value['uid'];
			}
		}
		if (empty($pages)) {
			$this->view->assign('hasTeasers', false);
			return;
		}
			// this reset the array-keys and thus removes the "holes",
			// created by the `unset($pages[$key]);` a few lines above
		$pages = array_values($pages);

		// DAM Files laden
		if(count($pagesUIDs)>0){
			$files = tx_dam_db::getReferencedFiles(
				'pages',
				implode(',', $pagesUIDs),
				'tx_form4_pages_teaser_teaserimg',
				'tx_dam_mm_ref',
				'pages.uid, tx_dam.file_name, tx_dam.file_path'
			);
// 			t3lib_div::debug(array($files,$pages));
			foreach($pages as $key => $value){
				$pages[$key]['hasImage'] = false;
				if($files['files'][$value['uid']]){
					$pages[$key]['hasImage'] = true;
				}
				$pages[$key]['file_name'] = $files['rows'][$value['uid']]['file_name'];
				$pages[$key]['file_path'] = $files['rows'][$value['uid']]['file_path'];
			}
		}
// 		t3lib_div::debug($pages);

		// Variablenübergabe ans Template
		$this->view->assign('pages', $pages);
		$this->view->assign('tag', $tagData['title']);
		$this->view->assign('hasTeasers', true);
		$this->view->assign('imageMargin', intval($this->settings['teaserImageWidth'])+10);
	}

	/**
	 * action showCloud
	 *
	 * @return void
	 */
	public function showCloudAction() {

		$tagsCount = max(intval($this->settings['tagsCount']),0);
		$contentObject = $this->configurationManager->getContentObject();
		$rows = $this->tagRepository->getTagsWithPageReferenceCount($tagsCount > 0 ? $tagsCount : '');
		
		if ($this->settings['enabledTagRecordsSortable']) {
			$rows = array_map(array($this,'makeTagRecordsSortable'),$rows);
				// sort by letters
			uasort($rows, 'Tx_Form4Tags_Utils::sortByTermAscending');
		}

		$rows = $this->_setWeightAndOrder($rows);

		$this->view->assign('tags', $rows);
	}

	protected function makeTagRecordsSortable($row) {
		return Tx_Form4Tags_Utils::getNaturalSortableString($row, 'title');
	}

	/**
	 * Set Weight and Order
	 * @param array $rows
	 * @return array
	 */
	private function _setWeightAndOrder($rows){
		$minsize = 1; // Mindestgroesse fuer CSS Klasse
		$maxsize = 9; // Hoechstgroesse fuer CSS Klasse
		$mincount = $maxcount = $rows[0]['counter']; // Zaehler fuer Grössenberechnung aus erstem ELement nehmen

		// groesste und kleinste Anzahl feststellen
		foreach ($rows as $row){
			if($row['counter']<$mincount) $mincount = $row['counter'];
			if($row['counter']>$maxcount) $maxcount = $row['counter'];
		}

		if($mincount == $maxcount){
			$maxcount = $maxcount + 1;
		}
		reset($rows);

		// CSS Klassen Zusatz definieren
		foreach ($rows as $key => $row){
			$rows[$key]['step'] = intval($minsize + (($maxcount-($maxcount-($row['counter']-$mincount)))*($maxsize-$minsize)/($maxcount-$mincount)));
		}

		return $rows;
	}

	/**
	 * action showTopics
	 *
	 * @return void
	 */
	public function showTopicsAction() {
		$topicsCount = intval($this->settings['topicsCount']);

		// Tags der aktuellen Seite laden
		$rows = $this->tagRepository->getTagsByPageUid($GLOBALS["TSFE"]->id);

// 		t3lib_div::debug($rows,'TOPICS');

		// Tags mit Wertigkeit versehen
		$tagUIDs = array();
		$tagWert = array();
		$wert = count($rows);
		foreach($rows AS $row){
			$tagUIDs[] = $row['uid'];
			$tagWert[$row['uid']] = $wert;
			$wert--;
		}

		// Seiten mit gleichen Tags laden
		if (empty($tagUIDs)) {
			$rows = array();
		} else {
			$rows = $this->tagRepository->getPageTagRelations(
				$tagUIDs, array($GLOBALS['TSFE']->id)
			);
		}
// 		t3lib_div::debug(array(
// 			$tagUIDs,
// 			$rows
// 		),'TOPICS');

		// Seitenwert berechnen
		$pagesData = array();
		foreach($rows AS $row){
			$pagesData[$row['pageUid']] += $tagWert[$row['tagUid']];
		}

		// Sortierung und Begrenzung
		asort($pagesData);
		$pagesData = array_reverse($pagesData, true);
		if($topicsCount>0){
			$pagesData = array_slice($pagesData, 0, $topicsCount, true);
		}
		$pagesData = array_keys($pagesData);

		$this->view->assign('pages', $pagesData);
	}
}
?>