<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2011 Kerstin Gerull <support@form4.de>, form4 GmbH & Co. KG
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/


/**
 *
 *
 * @package form4_tags
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License, version 3 or later
 *
 */
class Tx_Form4Tags_Domain_Repository_TagRepository extends Tx_Extbase_Persistence_Repository {

	/**
	 * @var Tx_Form4Tags_Controller_TagController a reference to the controller
	 */
	protected $_controller;
	/**
	* @var tslib_cObj a reference to the controller's contentObject
	*/
	protected $_contentObject;

	/**
	 * @var t3lib_db a reference to $GLOBALS['TYPO3_DB']
	 */
	protected $_db;

	/**
	 * @var tslib_fe a reference to $GLOBALS['TSFE']
	 */
	protected $_tsfe;

	/**
	 * @var string holding the valid sy_language_uid(s), eg. 0,-1
	 */
	protected $_sys_language_content;

	public function setController(Tx_Form4Tags_Controller_TagController & $controller) {
		$this->_controller = $controller;
		$this->_db = & $GLOBALS['TYPO3_DB'];
		$this->_tsfe = & $GLOBALS['TSFE'];
		if ($this->_tsfe->sys_language_contentOL) {
			// Sys language content is set to zero/-1 - and it is expected
			// that whatever routine processes the output will
			// OVERLAY the records with localized versions!
			$this->_sys_language_content = '0,-1';
		} else {
			$this->_sys_language_content = intval($this->_tsfe->sys_language_content);
		}
	}

	public function setContentObject(tslib_cObj & $contentObject) {
		$this->_contentObject = $contentObject;
	}

	function getPagesByTagUid($uid, $limit='') {
		$pages = $this->_db->exec_SELECTgetRows(
			'pages.*',
			'pages '.
				'LEFT JOIN ( tx_form4tags_pages_tags_mm, tx_form4tags_domain_model_tag ) '.
				'ON( pages.uid=tx_form4tags_pages_tags_mm.uid_local AND '.
				'tx_form4tags_pages_tags_mm.uid_foreign=tx_form4tags_domain_model_tag.uid )',
			'tx_form4tags_domain_model_tag.uid='.intval($uid).' AND '.
			'tx_form4tags_domain_model_tag.sys_language_uid IN('.$this->_sys_language_content.')'.
			$this->_contentObject->enableFields('tx_form4tags_domain_model_tag').
			$this->_contentObject->enableFields('pages'),
			'',
			'pages.SYS_LASTCHANGED DESC, pages.tstamp DESC',
			$limit
		);
		if (empty($pages)) {
			return $pages;
		}
		foreach($pages AS $key => $value){
				// Versioning preview:
			$this->_tsfe->sys_page->versionOL('pages', $value, TRUE);
				// Language overlay:
			if (
				$this->_tsfe->sys_language_contentOL &&
				$this->_tsfe->sys_language_uid > 0
			) {
				$value = $this->_tsfe->sys_page->getPageOverlay($value);
			}
			$pages[$key] = $value;
		}
		return $pages;
	}

	function getTagByUid($uid) {
		$data = $this->_db->exec_SELECTgetRows(
			'uid,pid,title',
			'tx_form4tags_domain_model_tag',
			'uid='.intval($uid).' AND '.
			'sys_language_uid IN('.$this->_sys_language_content.')'.
			$this->_contentObject->enableFields('tx_form4tags_domain_model_tag')
		);
		if (empty($data)) {
			return $data;
		}
		$data = $data[0];
		// Versioning preview:
		$this->_tsfe->sys_page->versionOL('tx_form4tags_domain_model_tag', $data, TRUE);

		// Language overlay:
		if ($this->_tsfe->sys_language_contentOL) {
			$data = $this->_tsfe->sys_page->getRecordOverlay(
				'tx_form4tags_domain_model_tag',
				$data,
				$this->_tsfe->sys_language_content,
				$this->_tsfe->sys_language_contentOL
			);
		}
		return $data;
	}

	function getTagsWithPageReferenceCount($limit='') {
		$rows = $this->_db->exec_SELECTgetRows(
			'tx_form4tags_domain_model_tag.uid, tx_form4tags_domain_model_tag.pid, tx_form4tags_domain_model_tag.title, COUNT(pages.uid) AS counter',
			'tx_form4tags_domain_model_tag '.
				'INNER JOIN tx_form4tags_pages_tags_mm ON tx_form4tags_domain_model_tag.uid=tx_form4tags_pages_tags_mm.uid_foreign '.
				'INNER JOIN pages ON tx_form4tags_pages_tags_mm.uid_local=pages.uid',
			'tx_form4tags_domain_model_tag.sys_language_uid IN('.$this->_sys_language_content.')'.
			$this->_contentObject->enableFields('tx_form4tags_domain_model_tag').
			$this->_contentObject->enableFields('pages'),
			'tx_form4tags_domain_model_tag.uid',
			'counter DESC',
			$limit
		);
		if (empty($rows)) {
			return $rows;
		}
		if (
			$this->_tsfe->sys_language_contentOL &&
			$this->_tsfe->sys_language_uid > 0
		) {
			foreach($rows as $index => $row) {
				$pages = $this->getPagesByTagUid($row['uid'],$limit);
				foreach($pages as $key => $page) {
					if (!$page['_PAGES_OVERLAY'] ) {
						unset($pages[$key]);
					}
				}
				if (empty($pages)) {
					unset($rows[$index]);
				} else {
						// Versioning preview:
					$this->_tsfe->sys_page->versionOL('tx_form4tags_domain_model_tag', $row, TRUE);
						// Language overlay:
					$row = $this->_tsfe->sys_page->getRecordOverlay(
						'tx_form4tags_domain_model_tag',
						$row,
						$this->_tsfe->sys_language_content,
						$this->_tsfe->sys_language_contentOL
					);
					$rows[$index] = $row;
				}
			}
		}
		return $rows;
	}

	function getTagsByPageUid($uid) {
// 		$this->_db->store_lastBuiltQuery = TRUE;
		$rows = $this->_db->exec_SELECTgetRows(
			'tx_form4tags_domain_model_tag.uid, tx_form4tags_domain_model_tag.pid',
			'pages '.
				'LEFT JOIN (tx_form4tags_pages_tags_mm, tx_form4tags_domain_model_tag) '.
				'ON ( pages.uid=tx_form4tags_pages_tags_mm.uid_local AND '.
				'tx_form4tags_pages_tags_mm.uid_foreign=tx_form4tags_domain_model_tag.uid )',
			'pages.uid='.intval($uid).
			' AND tx_form4tags_domain_model_tag.sys_language_uid IN('.$this->_sys_language_content.')'.
			$this->_contentObject->enableFields('tx_form4tags_domain_model_tag').
			$this->_contentObject->enableFields('pages'),
			'',
			'pages.sorting'
		);
// 		t3lib_div::debug(array($this->_db->debug_lastBuiltQuery));
// 		$this->_db->store_lastBuiltQuery = FALSE;
// 		t3lib_div::debug($rows,'getTagsByPageUid');
		foreach($rows AS $key => $row){
				// Versioning preview:
			$this->_tsfe->sys_page->versionOL('tx_form4tags_domain_model_tag', $row, TRUE);
				// Language overlay:
			if ($this->_tsfe->sys_language_contentOL) {
				$row = $this->_tsfe->sys_page->getRecordOverlay(
					'tx_form4tags_domain_model_tag',
					$row,
					$this->_tsfe->sys_language_content,
					$this->_tsfe->sys_language_contentOL
				);
			}
			$rows[$key] = $row;
		}
		return $rows;
	}
	function getPageTagRelations(array $tag_uids, array $exclude_pids) {
		$rows = $this->_db->exec_SELECTgetRows(
			'tx_form4tags_domain_model_tag.uid AS tagUid, pages.uid AS pageUid',
			'pages '.
				'LEFT JOIN (tx_form4tags_pages_tags_mm, tx_form4tags_domain_model_tag) '.
				'ON ( pages.uid=tx_form4tags_pages_tags_mm.uid_local AND '.
				'tx_form4tags_pages_tags_mm.uid_foreign=tx_form4tags_domain_model_tag.uid )',
			'tx_form4tags_domain_model_tag.uid IN('.implode(',', $tag_uids).') AND '.
			'pages.uid NOT IN('.implode(',', $exclude_pids).') AND '.
			'tx_form4tags_domain_model_tag.sys_language_uid IN('.$this->_sys_language_content.') '.
			$this->_contentObject->enableFields('tx_form4tags_domain_model_tag').
			$this->_contentObject->enableFields('pages')
		);
// 		t3lib_div::debug($rows,'getPageTagRelations');
		if (
			$this->_tsfe->sys_language_contentOL &&
			$this->_tsfe->sys_language_uid > 0
		) {
			foreach($rows as $index => $row) {
				$pages = $this->getPagesByTagUid($row['tagUid']);
// 				t3lib_div::debug($pages,'getPageTagRelations inner');
				foreach($pages as $key => $page) {
					if (!$page['_PAGES_OVERLAY'] ) {
						unset($rows[$index]);
					}
				}
			}
		}
// 		t3lib_div::debug($rows,'getPageTagRelations after');
			// this reset the array-keys and thus removes the "holes",
			// created by the `unset($rows[$index]);` a few lines above
		return array_values($rows);
	}
}
?>